<?php

namespace App\Http\Controllers\V1\Evidencia;

use App\Dto\EvidenciaDto;
use App\Dto\EvidenciaRegistroDto;
use App\Dto\EvidenciaRetoDto;
use App\Dto\HistorialRecompensaDto;
use App\Dto\NivelRetoDto;
use App\Enums\EstatusEvidencia;
use App\Http\Controllers\Controller;
use App\Http\Requests\CreateEvidenciaRequest;
use App\Models\Evidencia;
use App\Models\EvidenciaRegistro;
use App\Models\EvidenciaReto;
use App\Models\HistorialRecompensas;
use App\Models\NivelReto;
use App\Models\Reto;
use App\Services\EvidenciaService;
use App\Services\HistorialRecompensaService;
use App\Services\NivelService;
use App\Services\RetoService;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;

class EvidenciaController extends Controller
{
    protected $evidenciaService;
    protected $retoService;
    protected $nivelService;
    protected $historialService;

    public function __construct(EvidenciaService $evidenciaService, RetoService $retoService, 
    NivelService $nivelService, HistorialRecompensaService $historialService)
    {
        $this->evidenciaService = $evidenciaService;
        $this->retoService = $retoService;
        $this->nivelService = $nivelService;
        $this->historialService = $historialService;
    }

    /**
     * The index function retrieves all evidences and returns them as JSON data, handling errors if the
     * query is unsuccessful.
     * 
     * @return If the `` object's `success` property is false, an error response in JSON
     * format will be returned with a message indicating "Error al consultar las evidencias" and a
     * status code of 400. Otherwise, the data from the `` object will be returned.
     */
    public function index()
    {
        $resultQuery = $this->evidenciaService->getEvidenciasAll();
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ['Error al consultar las evidencias']]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * The function "show" retrieves detailed information about a specific evidence item in PHP.
     * 
     * @param id The `show` function in the code snippet is a method that takes an `` parameter.
     * This parameter is used to retrieve details of a specific evidence by calling the
     * `getEvidenciaDetail` method of the `` object. If the query is successful, it
     * returns
     * 
     * @return If the `->success` is false, the function will return a JSON response with
     * an error message stating "Error al consultar la evidencia" and a status code of 400.
     */
    public function show($id)
    {
        $resultQuery = $this->evidenciaService->getEvidenciaDetail($id);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ['Error al consultar la evidencia']]], 400);
        }
        return $resultQuery->data;
    }

    public function store(CreateEvidenciaRequest $request)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        try {
            DB::beginTransaction();
            $currentDateTime = Carbon::now();
            $dateNow = $currentDateTime->format('Y-m-d');

            $evidenciaDto = new EvidenciaDto();
            $evidenciaDto->comentario = $request->comentario;
            $evidenciaDto->participanteId = $request->participante_id;
            $evidenciaDto->userId = $request->user_id;
            $evidenciaDto->fileId = $request->file_id;
            $evidenciaDto->fechaCaptura = $dateNow;
            $resultCreate = $this->evidenciaService->addEvidencia($evidenciaDto);
            if (!$resultCreate->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al crear la evidencia"]]], 400);
            }
            $idEvidencia = $resultCreate->evidencia->id;
            //se asigna un registro de evidencia por cada producto
            foreach ($request->productos as $producto) {
                $idProducto = $producto['id'];
                $cantidad = $producto['cantidad'];
                $monto = $producto['monto'];
                $precioProducto = number_format($monto / $cantidad, 2);
                $evidenciaRegistro = new EvidenciaRegistroDto();
                $evidenciaRegistro->cantidad = $cantidad;
                $evidenciaRegistro->monto = $monto;
                $evidenciaRegistro->precioProducto = $precioProducto;
                $evidenciaRegistro->idProducto = $idProducto;
                $evidenciaRegistro->evidenciaId = $idEvidencia;
                $resultAddRegisterEvidence = $this->evidenciaService->addEvidenciaRegistro($evidenciaRegistro);
                if (!$resultAddRegisterEvidence->success) {
                    DB::rollback();
                    return response()->json(['error' => ['mensaje' => ["Error al asignar la evidencia"]]], 400);
                }
            }
            $listProductsId = array_column($request->productos, 'id');
            $resultRetosWithProductsAndPlayer = $this->retoService->getRetosWithProductsByPlayer($request->participante_id, $listProductsId);
            if (!$resultRetosWithProductsAndPlayer->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al asignar la evidencia al participante"]]], 400);
            }
            $listRetos = $resultRetosWithProductsAndPlayer->data;
            if (!empty($listRetos)) {
                foreach ($listRetos as $reto) {
                    $retoId = $reto->id;
                    $existsEvidenciaWithReto = $this->retoService->existAssingEvidenciaWithReto($retoId, $idEvidencia);
                    if (!$existsEvidenciaWithReto) {
                        $newRegistroReto = new EvidenciaRetoDto();
                        $newRegistroReto->idReto = $retoId;
                        $newRegistroReto->idEvidencia = $idEvidencia;
                        $resultAddEvidencia = $this->evidenciaService->addEvidenciaReto($newRegistroReto);
                        if (!$resultAddEvidencia) {
                            DB::rollback();
                            return response()->json(['error' => ['mensaje' => ["Error al asignar la evidencia a los retos"]]], 400);
                        }
                    }
                }
            }
            DB::commit();
            return response()->json([
                'message' => 'La evidencia se creo correctamente',
                'data' => $resultCreate->evidencia
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Error en el controller al crear la evidencia: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al crear la evidencia"]]], 400);
        }
    }

    public function evaluarEvidencia(Request $request, $id)
    {
        $data = $request->only('estatus', 'user_id');
        $validator = Validator::make($data, [
            'estatus' => 'required|string',
            'user_id' => 'required|integer|exists:users,id',
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            DB::beginTransaction();
            $evidenciaDto = new EvidenciaDto();
            $evidenciaDto->estatus = $request->estatus;
            $resultUpdateEvidencia = $this->evidenciaService->updateEvidenciaEstatusById($evidenciaDto, $id);
            if (!$resultUpdateEvidencia->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al actualizar el estatus de la evidencia"]]], 400);
            }
            if ($request->estatus !== EstatusEvidencia::APROBADO) {
                DB::commit();
                return response()->json([
                    'message' => 'Se evalúo correctamente la evidencia'
                ], Response::HTTP_OK);
            }
            $evidenciaFind = $resultUpdateEvidencia->evidencia;
            $idEvidencia = $evidenciaFind->id;
            $fechaCapturaEvidencia = $evidenciaFind->fechaCaptura;
            $idParticipante = $evidenciaFind->participanteId;

            //consultos niveles que esten relacionados a la evidencia tomando en cuenta la fecha de captura en
            // la que se creo la evidencia y que no tengan una recompensa asignada, la recompensa me indica si fue cumplido
            $resultNiveles = $this->nivelService->getNivelsWhereNotRecompensaByEvidencia($idEvidencia, $fechaCapturaEvidencia);
            if (!$resultNiveles->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al obtener información para asignar la evidencia"]]], 400);
            }
            $nivelesReto = $resultNiveles->data;
            foreach ($nivelesReto as $nivelReto) {
                $nivelRetoDto = new NivelRetoDto();
                $nivelRetoDto->idReto = $nivelReto->reto_detail['id'];
                $nivelRetoDto->meta = $nivelReto['meta'];
                $nivelRetoDto->tipoReto = $nivelReto->reto_detail['tipo_reto'];

                $resultCumpleNivelReto = $this->evidenciaService->isNivelRetoSuccessByEvidencias($nivelRetoDto, $idParticipante);
                if(!$resultCumpleNivelReto->success){
                    DB::rollback();
                    return response()->json(['error' => ['mensaje' => ["Error al procesar la evidencia en los retos del participante"]]], 400);
                }
                $isNivelRetoSuccess = $resultCumpleNivelReto->isNivelRetoSuccess;
                if ($isNivelRetoSuccess) {
                    $newHistorialDto = new HistorialRecompensaDto();
                    $newHistorialDto->participanteId = $idParticipante;
                    $newHistorialDto->nivelId = $nivelReto->id;
                    $newHistorialDto->userId = $request->user_id;
                    $resultCreate = $this->historialService->addHistorialRecompensa($newHistorialDto);
                    if (!$resultCreate->success) {
                        DB::rollback();
                        return response()->json(['error' => ['mensaje' => ["Error al crear la solicitud de entrega de recompensa"]]], 400);
                    }
                }
            }
            DB::commit();
            return response()->json([
                'message' => 'Se evalúo correctamente la evidencia',
                'data' => $resultUpdateEvidencia->evidencia
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Error en el controller al evaluar la envidencia: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al evaluar la evidencia"]]], 400);
        }
    }
}
