<?php

namespace App\Http\Controllers\V1\Historial;

use App\Dto\HistorialRecompensaDto;
use App\Http\Controllers\Controller;
use App\Services\EmailService;
use App\Services\HistorialRecompensaService;
use App\Services\NivelService;
use App\Services\PlayerService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;

class HistorialRecompensaController extends Controller
{

    protected $historialService;
    protected $emailService;
    protected $playerService;
    protected $nivelService;

    public function __construct(
        HistorialRecompensaService $historialService,
        EmailService $emailService,
        PlayerService $playerService,
        NivelService $nivelService
    ) {
        $this->historialService = $historialService;
        $this->emailService = $emailService;
        $this->playerService = $playerService;
        $this->nivelService = $nivelService;
    }

    public function index()
    {
        $resultQuery = $this->historialService->getHistorialRecompensasAll();
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las solicitudes de recompensa"]]], 400);
        }
        return $resultQuery->data;
    }


    /**
     * The function "show" retrieves and returns the details of a reward based on the provided ID.
     * 
     * @param id The `show` function takes an `` parameter, which is used to retrieve the details of
     * a reward by calling the `getRecompensaDetail` method of the `historialService`. If the query is
     * successful, it returns the data of the reward. If the query fails, it
     * 
     * @return If the `` object's `success` property is false, the function will return a
     * JSON response with an error message. Otherwise, it will return the data from the ``
     * object.
     */
    public function show($id)
    {
        $resultQuery = $this->historialService->getRecompensaDetail($id);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el detalle de solicitud recompensa"]]], 400);
        }
        return $resultQuery->data;
    }

    public function store(Request $request)
    {
        $data = $request->only('participante_id', 'nivel_id', 'user_id');
        $validator = Validator::make($data, [
            'participante_id' => 'required|integer|exists:participantes,id',
            'nivel_id' => 'required|integer|exists:nivel_retos,id',
            'user_id' => 'required|integer|exists:users,id',
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $newHistorialDto = new HistorialRecompensaDto();
            $newHistorialDto->participanteId = $request->participante_id;
            $newHistorialDto->nivelId = $request->nivel_id;
            $newHistorialDto->userId = $request->user_id;
            $resultCreate = $this->historialService->addHistorialRecompensa($newHistorialDto);
            if (!$resultCreate->success) {
                return response()->json(['error' => ['mensaje' => ["Error al crear la solicitud de entrega de recompensa"]]], 400);
            }
            return response()->json([
                'message' => 'Se creo la solicitud de recompensa correctamente',
                'data' => $resultCreate->historiaRecompensa
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller al crear la solicitud de recompensa: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al crear la solicitud de recompensa"]]], 400);
        }
    }

    public function update(Request $request, $id)
    {
        try {
            $data = $request->only(
                'user_id',
            );
            $validator = Validator::make($data, [
                'user_id' => 'required|integer|exists:users,id'
            ]);

            if ($validator->fails()) {
                return response()->json(['error' => $validator->messages()], 400);
            }
            DB::beginTransaction();
            $currentDateTime = Carbon::now();
            $dateNow = $currentDateTime->format('Y-m-d');
            $historialUpdateDto = new HistorialRecompensaDto();
            $historialUpdateDto->estatus = "entregado";
            $historialUpdateDto->userId = $request->user_id;
            $historialUpdateDto->fechaEntrega = $dateNow;
            $resultUpdateHistori = $this->historialService->updateHistorialRecompensa($historialUpdateDto, $id);
            if (!$resultUpdateHistori->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al procesar la entrega de la recompensa"]]], 400);
            }
            $participanteId = $resultUpdateHistori->historiaRecompensa->participanteId;
            $nivelId = $resultUpdateHistori->historiaRecompensa->nivelId;
            $resultParticipanteGet = $this->playerService->getPlayerByIdWithUserOnlyData($participanteId);
            if (!$resultParticipanteGet->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al obtener información del participante para notificar"]]], 400);
            }

            $participanteFind = $resultParticipanteGet->data;
            $resultNivelGet = $this->nivelService->getNivelFindById($nivelId);
            if (!$resultNivelGet->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al obtener información del nivel del reto"]]], 400);
            }
            
            $nombreNivelReto = $resultNivelGet->nivel->nombre;

            $details = [
                'email' => $participanteFind->usuario->email,
                'tienda' => $participanteFind->nombre_tienda,
                "nombre_participante" => $participanteFind->nombre_completo,
                'nombre_reto' => $nombreNivelReto,
                'fecha_entrega' => $dateNow,
                'direccion' => $participanteFind->direccion
            ];

            $sendEmailNotificacion = $this->emailService->SendEmailRecompensaApprove($details);
            if (!$sendEmailNotificacion) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al notificar al participante de la entrega de la recompensa"]]], 400);
            }
            DB::commit();
            return response()->json([
                'message' => 'Se entrego la recompensa correctamente',
                'data' => $resultUpdateHistori->historiaRecompensa
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al entregar la recompensa"]]], 400);
        }
    }
}
