<?php

namespace App\Http\Controllers\V1\Insignia;

use App\Dto\InsigniaDto;
use App\Http\Controllers\Controller;
use App\Http\Requests\CreateInsigniaRequest;
use App\Http\Requests\UpdateInsigniaRequest;
use App\Models\Insignia;
use App\Services\InsigniaService;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;

class InsigniaController extends Controller
{
    protected $insigniaService;

    public function __construct(InsigniaService $insigniaService)
    {
        $this->insigniaService = $insigniaService;
    }

    /**
     * The index function retrieves all insignias and returns them as JSON data, handling errors if the
     * query is unsuccessful.
     * 
     * @return the data from the `` if the query was successful. If the query was not
     * successful, it returns a JSON response with an error message "Error al consultar las insignias"
     * and a status code of 400.
     */
    public function index()
    {
        $resultQuery = $this->insigniaService->getInsigniasAll();
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las insignias"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * The function retrieves detailed information about an insignia by its ID and returns the data or
     * an error message if the query is unsuccessful.
     * 
     * @param id The `show` function takes an `` parameter as input. This function retrieves details
     * of an insignia by its ID using the `getInsigniaByIdDetail` method from the `insigniaService`. If
     * the query is successful, it returns the data of the insignia. If
     * 
     * @return the data from the `` object if the query was successful. If the query was
     * not successful, it returns a JSON response with an error message "Error al consultar la
     * insignia" and a status code of 400.
     */
    public function show($id)
    {
        $resultQuery = $this->insigniaService->getInsigniaByIdDetail($id);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar la insignia"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * The function `store` creates a new insignia using data from a request and returns a JSON
     * response with the result.
     * 
     * @param CreateInsigniaRequest request The `store` function you provided seems to be a part of a
     * Laravel controller method for storing a new "Insignia" based on the data received in the
     * `CreateInsigniaRequest` request.
     * 
     * @return a JSON response. If the creation of the insignia is successful, it returns a JSON
     * response with a success message and the data of the created insignia. If there is an error
     * during the creation process, it returns a JSON response with an error message indicating the
     * failure to create the insignia.
     */
    public function store(CreateInsigniaRequest $request)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        $newItem = new InsigniaDto();
        $newItem->nombre = $request->nombre;
        $newItem->objetivo = $request->objetivo;
        $newItem->userId = $request->user_id;
        $newItem->fileId = $request->file_id;
        $newItem->recompensaId = $request->recompensa_id;
        $resultCreate = $this->insigniaService->addInsignia($newItem);
        if (!$resultCreate->success) {
            return response()->json(['error' => ['mensaje' => ["Error al crear insignia"]]], 400);
        }
        return response()->json([
            'message' => 'La insignia se creó correctamente',
            'data' => $resultCreate->insigniaDto
        ], Response::HTTP_OK);
    }

    /**
     * This PHP function updates an insignia with the data provided in the request.
     * 
     * @param Request request The `update` function in the provided code snippet is responsible for
     * updating an "Insignia" entity based on the data provided in the request. Here's a breakdown of
     * the code:
     * @param id The `id` parameter in the `update` function represents the unique identifier of the
     * item or record that you want to update. It is typically used to identify the specific item in
     * the database that needs to be updated. In this case, it seems like you are updating an
     * "Insignia"
     * 
     * @return a JSON response. If the validation fails, it returns an error message with the
     * validation messages. If the update operation is successful, it returns a success message along
     * with the updated insignia data.
     */
    public function update(UpdateInsigniaRequest $request, $id)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        $itemDto = new InsigniaDto();
        $itemDto->id = $id;
        $itemDto->nombre = $request->nombre;
        $itemDto->objetivo = $request->objetivo;
        $itemDto->estatus = $request->estatus;
        $itemDto->fileId = $request->file_id;
        $itemDto->recompensaId = $request->recompensa_id;

        $resultUpdate = $this->insigniaService->updateInsignia($itemDto, $id);
        if (!$resultUpdate->success) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar la insignia"]]], 400);
        }
        return response()->json([
            'message' => 'La insignia se actualizo correctamente',
            'data' => $resultUpdate->insigniaDto
        ], Response::HTTP_OK);
    }

    /**
     * The function `destroy` in PHP deletes an insignia by its ID after checking for related
     * relationships and returns appropriate JSON responses.
     * 
     * @param id The `destroy` function you provided seems to be a part of a Laravel controller
     * responsible for deleting a specific record from the database based on the given `id`.
     * 
     * @return a JSON response with a success message 'Se eliminó la insignia correctamente' and a
     * status code of 200 (HTTP_OK) if the insignia is successfully deleted. If there is an error
     * during the deletion process, it returns a JSON response with an error message indicating the
     * issue encountered.
     */
    public function destroy($id)
    {
        $insigniaFind = Insignia::find($id);
        if (!$insigniaFind)
            return response()->json(['error' => ['mensaje' => ["La insignia no existe"]]], 400);
        $existRelacionesInsignia = $insigniaFind->secureDelete("niveles_retos");
        if ($existRelacionesInsignia)
            return response()->json(['error' => ['mensaje' => ["La insignia ha sido asignada a un nivel."]]], 400);

        $result = $this->insigniaService->removeInsigniaById($id);
        if (!$result) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el insignia"]]], 400);
        }
        return response()->json([
            'message' => 'Se eliminó la insignia correctamente'
        ], Response::HTTP_OK);
    }
}
