<?php

namespace App\Http\Controllers\V1\Perfil;

use App\Dto\PlayerDto;
use App\Dto\UserDto;
use App\Http\Controllers\Controller;
use App\Models\User;
use App\Services\AuthService;
use App\Services\PlayerService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;

class PerfilController extends Controller
{
    protected $playerService;
    protected $authService;

    public function __construct(PlayerService $playerService, AuthService $authService)
    {
        $this->playerService = $playerService;
        $this->authService = $authService;
    }

    /**
     * The function `getPerfil` retrieves a user's profile information along with their associated
     * participant data in PHP.
     * 
     * @param userId The `getPerfil` function is designed to retrieve the profile information for a
     * user with the given `userId`. The function attempts to find the user record along with their
     * associated `participante` relationship using Eloquent ORM.
     * 
     * @return The `getPerfil` function is returning the user data along with their associated
     * `participante` data if found based on the provided ``. If an error occurs during the
     * query execution, it will return a JSON response with an error message indicating "Error al
     * consultar el perfil" and a status code of 400.
     */
    public function getPerfil($userId)
    {
        $queryResult = $this->authService->getPerfilUser($userId);
        if(!$queryResult->success){
            return response()->json(['error' => ['mensaje' => ["Error al consultar tu perfil"]]], 400);
        }
        return $queryResult->data;
    }


    /**
     * The function `updatePerfil` updates user profile information including email, name, address, and
     * store details, handling validation and error cases.
     * 
     * @param Request request The `updatePerfil` function is a PHP function that handles updating user
     * profile information based on the data provided in the request. Let's break down the function and
     * its functionality:
     * 
     * @return The function `updatePerfil` is returning a JSON response. If the validation fails, it
     * returns an error message with the validation messages. If the update process is successful, it
     * returns a success message along with the updated user and participant data in the JSON format.
     * If an exception occurs during the update process, it returns an error message indicating the
     * failure to update the profile.
     */
    public function updatePerfil(Request $request)
    {
        //Validamos los datos
        $data = $request->only(
            'email',
            'nombre_tienda',
            'direccion',
            'nombre_completo',
            'localidad',
            'estado',
            'ciudad',
            'user_id',
            'file_id'
        );
        $validator = Validator::make($data, [
            'email' => 'nullable|email|unique:users',
            'nombre_tienda' => 'nullable|max:400|string',
            'direccion' => 'nullable|string',
            'nombre_completo' => 'required|max:400|string',
            'localidad' => 'nullable|max:400|string',
            'estado' => 'nullable|max:400|string',
            'ciudad' => 'nullable|max:400|string',
            'user_id' => 'required|integer|exists:users,id',
            'file_id' => 'integer|required|exists:files,id',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            DB::beginTransaction();
            $nombreTienda = $request->nombre_tienda ?? "";
            $direccion = $request->direccion ?? "";
            $nombreCompleto = $request->nombre_completo ?? "";
            $localidad = $request->localidad ?? "";
            $estado = $request->estado ?? "";
            $ciudad = $request->ciudad ?? "";
            $fileId = $request->file_id ?? "";
            $emailData = $request->email ?? "";

            $isAdminResult = $this->authService->isRoleAdmin($request->user_id);
            if (!$isAdminResult->success) {
                return response()->json(['error' => ['mensaje' => ["Error al procesar la información del usuario"]]], 400);
            }
            $isRolAdmin = $isAdminResult->data;
            $userDto = new UserDto();
            $userDto->name = $request->nombre_completo;
            $userDto->email = $emailData;
            $resultUpdateUser = $this->authService->updateUserById($userDto, $request->user_id);
            if (!$resultUpdateUser->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al guardar el nombre"]]], 400);
            }
            $userUpdate = $resultUpdateUser->user;
            $email = $userUpdate->email;

            if ($isRolAdmin) {
                DB::commit();
                return response()->json([
                    'message' => 'Se actualiazó correctamente el usuario',
                    'data' => [
                        "email" => $email,
                        "nombre_completo" => $userUpdate->name,
                    ]
                ], Response::HTTP_OK);
            }

            $playerDto = new PlayerDto($nombreCompleto, $nombreTienda, $direccion, $localidad, $estado, $ciudad, "", $fileId, "", "");
            $resultUpdate = $this->playerService->updatePlayerByUserId($playerDto, $request->user_id);
            if (!$resultUpdate->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al guardar la información del perfil"]]], 400);
            }

            $participante = $resultUpdate->player;
            DB::commit();
            return response()->json([
                'message' => 'Se actualiazó correctamente el usuario',
                'data' => [
                    "email" => $email,
                    "localidad" => $participante->localidad ?? '',
                    "nombre_completo" => $participante->nombreCompleto ?? $userUpdate->name,
                    "estado" => $participante->estado ?? '',
                    "ciudad" => $participante->ciudad ?? '',
                    "direccion" => $participante->direccion ?? '',
                    "nombre_tienda" => $participante->nombreTienda ?? ''
                ]
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            DB::rollback();
            Log::error('Error en el controller al actualizar el perfil: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el perfil"]]], 400);
        }
    }
}
