<?php

namespace App\Http\Controllers\V1\Recompensa;

use App\Dto\RecompensaDto;
use App\Http\Controllers\Controller;
use App\Http\Requests\CreateRecompensaRequest;
use App\Http\Requests\UpdateRecompensaRequest;
use App\Models\Recompensa;
use App\Services\RecompensaService;
use Symfony\Component\HttpFoundation\Response;

class RecompensaController extends Controller
{

    protected $recompensaService;

    public function __construct(RecompensaService $recompensaService)
    {
        $this->recompensaService = $recompensaService;
    }

    /**
     * The function `getRecompensasActivas` retrieves active rewards and returns them as JSON data or
     * an error message if the retrieval fails.
     * 
     * @return The function `getRecompensasActivas` is returning either an error response in JSON
     * format with a message indicating an error occurred while querying the rewards, or it is
     * returning the data of active rewards if the query was successful.
     */
    public function getRecompensasActivas()
    {
        $result = $this->recompensaService->getRecompensasActive();
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las recompensas"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function retrieves all rewards and returns them as JSON, handling errors if the retrieval
     * fails.
     * 
     * @return If the `` object's `success` property is false, the function will return a JSON
     * response with an error message "Error al consultar las recompensas" and a status code of 400.
     * Otherwise, it will return the data from the `` object.
     */
    public function index()
    {
        $result = $this->recompensaService->getRecompensasAll();
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las recompensas"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function "show" retrieves a reward by its ID and returns the data if successful, or an error
     * message if the retrieval fails.
     * 
     * @param id The `show` function in the code snippet is a method that takes an `` parameter.
     * This function retrieves a reward by its ID using the `getRecompensaById` method from the
     * ``. If the retrieval is successful, it returns the reward data. Otherwise,
     * 
     * @return If the `->success` is false, an error response in JSON format with a message
     * "Error al consultar la recompensa" will be returned with a status code of 400. Otherwise, the
     * data from the `getRecompensaById` method will be returned.
     */
    public function show($id)
    {
        $result = $this->recompensaService->getRecompensaById($id);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar la recompensa"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function `store` creates a new reward with default values if validation passes, otherwise
     * returns an error response.
     * 
     * @param CreateRecompensaRequest request The `store` function you provided seems to be handling
     * the creation of a "Recompensa" (reward) based on the data received in the
     * `CreateRecompensaRequest` object.
     * 
     * @return The function `store` is returning a JSON response. If the validation fails, it returns
     * an error message with the validation messages. If the creation of the reward is successful, it
     * returns a success message along with the created reward data.
     */
    public function store(CreateRecompensaRequest $request)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        $itemDto = new RecompensaDto();
        $itemDto->nombre = $request->nombre;
        $itemDto->descripcionPremio = $request->descripcion_premio;
        $itemDto->metaCanje = $request->meta_canje;
        $itemDto->fileId = $request->file_id;

        $result = $this->recompensaService->addRecompensa($itemDto);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al crear el producto."]]], 400);
        }
        return response()->json([
            'message' => 'La recompensa se creó correctamente',
            'data' => $result->recompensa
        ], Response::HTTP_OK);
    }


    /**
     * This PHP function updates a reward based on the provided request data and returns a JSON
     * response with success or error messages.
     * 
     * @param UpdateRecompensaRequest request The `update` function you provided seems to be updating a
     * reward based on the data received in the `UpdateRecompensaRequest` object and the ``
     * parameter.
     * @param id The `id` parameter in the `update` function is used to identify the specific
     * recompensa (reward) that needs to be updated. It is typically the unique identifier of the
     * recompensa in the database, such as the primary key. This parameter helps to target the correct
     * recompensa
     * 
     * @return The `update` function is returning a JSON response. If the validation fails, it returns
     * an error response with the validation messages. If the update operation is successful, it
     * returns a success message along with the updated recompensa data.
     */
    public function update(UpdateRecompensaRequest $request, $id)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        $itemDto = new RecompensaDto();
        $itemDto->id = $id;
        $itemDto->nombre = $request->nombre;
        $itemDto->descripcionPremio = $request->descripcion_premio;
        $itemDto->metaCanje = $request->meta_canje;
        $itemDto->fileId = $request->file_id;
        $itemDto->estatus = $request->estatus;

        $result = $this->recompensaService->updateRecompensa($itemDto, $id);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar la recompensa"]]], 400);
        }
        return response()->json([
            'message' => 'La recompensa se actualizó correctamente',
            'data' => $result->recompensa
        ], Response::HTTP_OK);
    }


    public function destroy($id)
    {
        $recompensaDelete = Recompensa::find($id);
        if (!$recompensaDelete)
            return response()->json(['error' => ['mensaje' => ["La recompensa no existe"]]], 400);

        $result = $this->recompensaService->removeRecompensaById($id);
        if (!$result) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar la recompensa"]]], 400);
        }
        return response()->json([
            'message' => 'Se eliminó la recompensa correctamente'
        ], Response::HTTP_OK);
    }
}
