<?php

namespace App\Http\Controllers\V1\Reto;

use App\Dto\NivelDto;
use App\Dto\RetoDto;
use App\Http\Controllers\Controller;
use App\Http\Requests\CreateRetoRequest;
use App\Http\Requests\UpdateRetoRequest;
use App\Models\Reto;
use App\Services\NivelService;
use App\Services\RetoService;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;


class RetoController extends Controller
{

    protected $retoService;
    protected $nivelService;

    public function __construct(RetoService $retoService, NivelService $nivelService)
    {
        $this->retoService = $retoService;
        $this->nivelService = $nivelService;
    }

    /**
     * The index function retrieves all challenges and returns the data or an error message if the
     * retrieval fails.
     * 
     * @return If the `` object's `success` property is false, the function will return a JSON
     * response with an error message "Error al consultar los retos" and a status code of 400.
     * Otherwise, it will return the data from the `` object.
     */
    public function index()
    {
        $result = $this->retoService->getRetosAll();
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los retos"]]], 400);
        }
        return $result->data;
    }

    /**
     * This PHP function retrieves challenges by participant type and returns the data or an error
     * message if unsuccessful.
     * 
     * @param typeReto The `typeReto` parameter in the `retosByTypeParticipante` function is used to
     * specify the type of challenge or task that the function will retrieve from the `retoService`.
     * The function then checks if the retrieval was successful and returns the data if it was, or an
     * 
     * @return the data of the retos (challenges) of a specific type for a participant. If the
     * retrieval of the data is successful, it returns the data. If there is an error in retrieving the
     * data, it returns a JSON response with an error message indicating "Error al consultar los retos"
     * (Error while querying the challenges) with a status code of 400.
     */
    public function retosByTypeParticipante($typeReto)
    {
        $result = $this->retoService->getRetosByTypePlayer($typeReto);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los retos"]]], 400);
        }
        return $result->data;
    }


    /**
     * The function "show" retrieves a challenge by its ID and returns the challenge data or an error
     * message if the retrieval fails.
     * 
     * @param id The `show` function takes an `` parameter, which is used to retrieve a specific
     * "reto" (challenge) by its ID. The function calls the `getRetoById` method of the ``
     * object to fetch the data for the specified ID. If the
     * 
     * @return If the `->success` is `false`, the function will return a JSON response with an
     * error message "Error al consultar el reto" and a status code of 400. Otherwise, it will return
     * the data from ``.
     */
    public function show($id)
    {

        $result = $this->retoService->getRetoById($id);
        if (!$result->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el reto"]]], 400);
        }
        return $result->data;
    }

    /**
     * The function `store` creates a new challenge with associated products and levels, handling
     * validation and database transactions in a Laravel PHP application.
     * 
     * @param CreateRetoRequest request The `store` function you provided seems to handle the creation
     * of a "Reto" along with its associated products and levels. Here's a breakdown of the parameters
     * used in the function:
     * 
     * @return a JSON response. If the validation fails, it returns an error message with the
     * validation messages. If an error occurs during the creation of the challenge or any related
     * entities, it returns an error message. If everything is successful, it returns a success message
     * along with the created challenge data.
     */
    public function store(CreateRetoRequest $request)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        try {
            $retoDto = new RetoDto();
            $retoDto->nombre = $request->nombre;
            $retoDto->descripcion = $request->descripcion;
            $retoDto->tipoParticipante = $request->tipo_participante;
            $retoDto->tipoReto = $request->tipo_reto;
            $retoDto->userId = $request->user_id;
            $retoDto->fileId = $request->file_id;

            DB::beginTransaction();
            $resultReto = $this->retoService->addReto($retoDto);
            if (!$resultReto->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al crear el reto"]]], 400);
            }
            $idReto = $resultReto->reto->id;

            foreach ($request->productos as $valor) {
                $idProducto = $valor['id'];
                $resultCreateProduct = $this->retoService->addRetoToProduct($idReto, $idProducto);
                if (!$resultCreateProduct->success) {
                    DB::rollback();
                    return response()->json(['error' => ['mensaje' => ["Error al asignar el reto al producto, verifica tus datos."]]], 400);
                }
            }

            foreach ($request->niveles as $valor) {
                $nivelDto = new NivelDto();
                $nivelDto->nombre = $valor['nombre'];
                $nivelDto->meta = $valor['meta'];
                $nivelDto->puntos = $valor['puntos'];
                $nivelDto->fechaInicio = $valor['fecha_inicio'];
                $nivelDto->fechaFin = $valor['fecha_fin'];
                $nivelDto->insigniaId = $valor['insignia_id'];
                $nivelDto->retoId = $idReto;
                $resultCreateNivel = $this->nivelService->addNivelReto($nivelDto);
                if (!$resultCreateNivel->success) {
                    DB::rollback();
                    return response()->json([
                        'error' => [
                            'mensaje' => [
                                "Error al asignar el nivel {$nivelDto->nombre} al reto, verifica tus datos."
                            ]
                        ]
                    ], 400);
                }
            }
            DB::commit();
            return response()->json([
                'message' => 'El reto se creó correctamente',
                'data' => $resultReto->reto
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller al crear el reto: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al crear el reto"]]], 400);
        }
    }


    /**
     * The function `update` in PHP updates a challenge with its details, levels, and associated
     * products, handling various error scenarios.
     * 
     * @param UpdateRetoRequest request The `update` function you provided seems to handle the updating
     * of a "Reto" entity along with its related data such as levels and products. Here's a breakdown
     * of the process:
     * @param idReto The `idReto` parameter in the `update` function is used to identify the specific
     * reto (challenge or task) that needs to be updated. It is likely an identifier, such as an
     * integer or string, that uniquely identifies the reto within your system. This parameter is used
     * to
     * 
     * @return a JSON response. In case of success, it returns a message indicating that the challenge
     * was updated correctly along with the updated challenge data. If there are any errors during the
     * update process, it returns an error message with details about the specific error encountered.
     */
    public function update(UpdateRetoRequest $request, $idReto)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }
        try {
            DB::beginTransaction();
            $retoDto = new RetoDto();
            $retoDto->nombre = $request->nombre;
            $retoDto->descripcion = $request->descripcion;
            $retoDto->tipoReto = $request->tipo_reto;
            $retoDto->fileId = $request->file_id;

            $resultUpdateReto = $this->retoService->updateReto($retoDto, $idReto);
            if (!$resultUpdateReto->success) {
                DB::rollback();
                return response()->json(['error' => ['mensaje' => ["Error al actualizar el reto"]]], 400);
            }

            $resultDeleteNiveles = $this->nivelService->removeListNivelsByReto($idReto, $request->niveles);
            if (!$resultDeleteNiveles->success) {
                $message = $resultDeleteNiveles->data ? "Error al eliminar el nivel {$resultDeleteNiveles->data} del reto, verifica tus datos." : "Error al intentar actualizar los niveles del reto.";
                DB::rollback();
                return response()->json([
                    'error' => [
                        'mensaje' => [
                            $message
                        ]
                    ]
                ], 400);
            }

            $resultNiveles = $this->nivelService->createOrUpdateListNivels($request->niveles, $idReto);
            if (!$resultNiveles) {
                DB::rollback();
                return response()->json([
                    'error' => [
                        'mensaje' => [
                            "Error al intentar actualizar los niveles del reto."
                        ]
                    ]
                ], 400);
            }

            $resultRemoveRelationWithProducts = $this->retoService->removeProductByRetoRelation($idReto, $request->productos);
            if (!$resultRemoveRelationWithProducts) {
                DB::rollback();
                return response()->json([
                    'error' => [
                        'mensaje' => [
                            "Error al intentar desvincular los productos del reto."
                        ]
                    ]
                ], 400);
            }

            foreach ($request->productos as $valor) {
                $idProducto = $valor['id'];
                $resultExistProductInReto = $this->retoService->existAssingProductWithReto($idReto, $idProducto);
                if (!$resultExistProductInReto) {
                    $resultCreateProduct = $this->retoService->addRetoToProduct($idReto, $idProducto);
                    if (!$resultCreateProduct->success) {
                        DB::rollback();
                        return response()->json(['error' => ['mensaje' => ["Error al asignar el reto al producto, verifica tus datos."]]], 400);
                    }
                }
            }
            DB::commit();
            return response()->json([
                'message' => 'El reto se actualizo correctamente',
                'data' => $resultUpdateReto->reto
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller al actualizar el reto: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el reto"]]], 400);
        }
    }

    /**
     * The function `asignReto` assigns participants to a challenge in PHP, handling validation,
     * database transactions, and error logging.
     * 
     * @param Request request The `asignReto` function is responsible for assigning participants to a
     * challenge. Let me explain the code step by step:
     * 
     * @return The function `asignReto` is returning a JSON response. In case of validation failure, it
     * returns an error response with the validation messages. If there is an error during the process,
     * it returns an error response with a message indicating the issue. If the process is successful,
     * it returns a success response with a message confirming that the participants have been assigned
     * to the challenge correctly.
     */
    public function asignReto(Request $request)
    {
        $data = $request->only(
            'reto_id',
            'participantes'
        );
        $validator = Validator::make($data, [
            'reto_id' => 'required|integer|exists:retos,id',
            'participantes' => 'sometimes|array',
            'participantes.*.participante_id' =>  'required|integer'
        ]);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            DB::beginTransaction();
            $idReto = $request->reto_id;
            $idFilterNotPlayers = array_column($request->all(), 'participante_id');
            $resultRemovePlayers = $this->retoService->removePlayersByRetoRelation($idReto, $idFilterNotPlayers);
            if (!$resultRemovePlayers) {
                DB::rollback();
                return response()->json([
                    'error' => [
                        'mensaje' => [
                            "Error al intentar desvincular los participantes del reto."
                        ]
                    ]
                ], 400);
            }
            if (!empty($request->participantes)) {
                foreach ($request->participantes as $valor) {
                    $participanteId = $valor['participante_id'];
                    $idReto = $request->reto_id;
                    $exitsAsignacionReto = $this->retoService->existAssingPlayerWithReto($idReto, $participanteId);
                    if (!$exitsAsignacionReto) {
                        $resultRetoPlayer = $this->retoService->addRetoToPlayer($idReto, $participanteId);
                        if (!$resultRetoPlayer->success) {
                            DB::rollback();
                            return response()->json(['error' => ['mensaje' => ["Error al asignar el reto al participante, verifica tus datos."]]], 400);
                        }
                    }
                }
            }
            DB::commit();
            return response()->json([
                'message' => 'Se asignaron correctamente los participantes al reto.'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller de asignar retos participantes: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al asignar los retos"]]], 400);
        }
    }

    /**
     * The function `destroy` in this PHP code snippet deletes a "Reto" object after checking for
     * related data and handling any errors.
     * 
     * @param id The `destroy` function in the provided code snippet is responsible for deleting a
     * "Reto" (challenge) by its ID. The function follows these steps:
     * 
     * @return a JSON response with a success message if the reto is deleted successfully. If there are
     * any errors during the deletion process, it returns a JSON response with an error message.
     */
    public function destroy($id)
    {
        try {
            $retoDelete = Reto::find($id);
            if (!$retoDelete)
                return response()->json(['error' => ['mensaje' => ["El reto no existe"]]], 400);

            $existRelacionesReto = $retoDelete->secureDelete("participantes");
            if ($existRelacionesReto)
                return response()->json(['error' => ['mensaje' => ["El reto ha sido asignado"]]], 400);

            $resultRemoveReto = $this->retoService->removeRetoById($id);
            if (!$resultRemoveReto) {
                return response()->json(['error' => ['mensaje' => ["Error al eliminar el reto"]]], 400);
            }

            return response()->json([
                'message' => 'Se eliminó el reto correctamente',
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller al eliminar el reto: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el reto"]]], 400);
        }
    }
}
