<?php

namespace App\Http\Controllers\V1\Reto;

use App\Http\Controllers\Controller;
use App\Models\Canje;
use App\Models\HistorialRecompensas;
use App\Services\CanjeService;
use App\Services\EvidenciaService;
use App\Services\HistorialRecompensaService;
use App\Services\InsigniaService;
use App\Services\NivelService;
use App\Services\PlayerService;
use App\Services\ProductService;
use App\Services\RetoService;
use Illuminate\Support\Facades\Log;
use Symfony\Component\HttpFoundation\Response;

class RetoParticipanteController extends Controller
{
    protected $retoService;
    protected $evidenciaService;
    protected $playerService;
    protected $insigniaService;
    protected $productService;
    protected $nivelService;
    protected $historialRecompensaService;
    protected $canjeService;

    public function __construct(
        RetoService $retoService,
        EvidenciaService $evidenciaService,
        PlayerService $playerService,
        InsigniaService $insigniaService,
        ProductService $productService,
        NivelService $nivelService,
        HistorialRecompensaService $historialRecompensaService,
        CanjeService $canjeService
    ) {
        $this->retoService = $retoService;
        $this->evidenciaService = $evidenciaService;
        $this->playerService = $playerService;
        $this->insigniaService = $insigniaService;
        $this->productService = $productService;
        $this->nivelService = $nivelService;
        $this->historialRecompensaService = $historialRecompensaService;
        $this->canjeService = $canjeService;
    }

    /**
     * The function retosVigentes retrieves active challenges for a participant and returns the data or
     * an error message if the query fails.
     * 
     * @param idParticipante The function `retosVigentes` takes a parameter `` which
     * represents the ID of a participant. This function retrieves and returns the active challenges
     * for the specified participant. If the query to fetch the active challenges is successful, it
     * returns the data containing the active challenges. Otherwise
     * 
     * @return The function `retosVigentes` is returning the data of the retos vigentes (current
     * challenges) for a given participant ID. If the query to retrieve the data is successful, it
     * returns the data. If the query is not successful, it returns a JSON response with an error
     * message indicating "Error al consultar tus retos vigentes" (Error while querying your current
     * challenges) and
     */
    public function retosVigentes($idParticipante)
    {
        $resultQuery = $this->retoService->getRetosVigentesByPlayer($idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus retos vigentes"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * The function retrieves challenges assigned to a participant and returns the data or an error
     * message if the query is unsuccessful.
     * 
     * @param idParticipante The `idParticipante` parameter in the `getRetosAsignadosParticipante`
     * function is used to specify the ID of the participant for whom we want to retrieve assigned
     * challenges. This function likely queries the `retoService` to get a list of challenges assigned
     * to the participant
     * 
     * @return The function `getRetosAsignadosParticipante` is returning the data of the retos
     * (challenges) assigned to a participant with the given ``. If the query to
     * retrieve the retos is successful, it returns the data of the retos. If the query is not
     * successful, it returns a JSON response with an error message indicating "Error al consultar mis
     */
    public function getRetosAsignadosParticipante($idParticipante)
    {
        $resultQuery = $this->retoService->getRetosByPlayer($idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar mis retos"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * The function `getRetoDetalleParticipante` retrieves the details of a participant in a challenge.
     * 
     * @param idParticipante The `idParticipante` parameter is the unique identifier of the participant
     * for whom you want to retrieve the details of a specific challenge (`idReto`). This function
     * `getRetoDetalleParticipante` takes two parameters: `idParticipante` and `idReto`, and
     * @param idReto The `idReto` parameter in the `getRetoDetalleParticipante` function represents the
     * unique identifier of the challenge or "reto" for which you want to retrieve details for a
     * specific participant. This parameter is used to query the details of a challenge for a
     * particular participant.
     * 
     * @return the data obtained from the `` if the query was successful. If the query was
     * not successful, it returns a JSON response with an error message indicating "Error al consultar
     * el detalle del reto" and a status code of 400.
     */
    public function getRetoDetalleParticipante($idParticipante, $idReto)
    {
        $resultQuery = $this->retoService->getRetoDetailByPlayer($idReto, $idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el detalle del reto"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * This PHP function retrieves evidences for a given participant ID.
     * 
     * @param idParticipante The `getEvidencias` function takes an `` parameter, which
     * is used to retrieve evidences associated with a specific participant. The function calls the
     * `getEvidenciasByPlayer` method from the `evidenciaService` and returns the data if the query is
     * 
     * @return The function `getEvidencias` is returning either an error response in JSON format with a
     * message indicating an error in querying the evidences of the participant, or it is returning the
     * data obtained from the query if the operation was successful.
     */
    public function getEvidencias($idParticipante)
    {
        $resultQuery = $this->evidenciaService->getEvidenciasByPlayer($idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las evidencias del participante"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * This PHP function retrieves rewards associated with a participant by querying a service and
     * handles potential errors.
     * 
     * @param idParticipante The `getRecompensas` function you provided seems to be a part of a PHP
     * class or controller. It appears to be retrieving rewards for a participant based on their ID.
     * 
     * @return The `getRecompensas` function returns either a JSON response with an error message if
     * there was an issue querying the rewards, or it returns the data containing rewards if the query
     * was successful.
     */
    public function getRecompensas($idParticipante)
    {
        try {
            $resultQuery = $this->historialRecompensaService->getRecompensaHasInsigniaByPlayer($idParticipante);
            if (!$resultQuery->success) {
                return response()->json(['error' => ['mensaje' => ["Error al consultar tus recompensas"]]], 400);
            }
            return $resultQuery->data;
        } catch (\Exception $e) {
            Log::error('Error en el controller al consultar recompensas de insignias del participante: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus recompensas, contacta al administrador"]]], 400);
        }
    }

    /**
     * The function retrieves the total sales of products by a participant and returns the result or an
     * error message if the query is unsuccessful.
     * 
     * @param idParticipante The `getTotalVentaProductosByParticipante` function seems to be retrieving
     * the total sales of products by a participant identified by the `` parameter. If
     * the query to `productService->getTotalVentaProductosByPlayer()` is successful, it
     * returns
     * 
     * @return the result of the query to get the total sales of products by a participant. If the
     * query is successful, it returns the resultQuery which contains the total sales data. If the
     * query is not successful, it returns a JSON response with an error message indicating that there
     * was an error while querying the sales data.
     */
    public function getTotalVentaProductosByParticipante($idParticipante)
    {
        $resultQuery = $this->productService->getTotalVentaProductosByPlayer($idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus datos de venta."]]], 400);
        }
        return $resultQuery->data;
    }

    public function getTotalVentaProductosGlobal()
    {
        $resultQuery = $this->productService->getTotalVentaProductos();
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el historia de venta de los productos."]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * The function `getTotalPuntosByParticipante` retrieves total points, weekly points, and monthly
     * points for a participant and returns the data in a JSON response.
     * 
     * @param idParticipante The `getTotalPuntosByParticipante` function is designed to retrieve and
     * return the total points, weekly points, and monthly points for a given participant based on
     * their `idParticipante`.
     * 
     * @return a JSON response with the total points, points for the week, and points for the month for
     * a specific participant. If there is an error during the process, it will return a JSON response
     * with an error message.
     */
    public function getTotalPuntosByParticipante($idParticipante)
    {
        try {
            $resultPointsTotal = $this->nivelService->getPointsTotalByPlayer($idParticipante);
            if (!$resultPointsTotal->success) {
                return response()->json(['error' => ['mensaje' => ["Error al consultar tus puntos."]]], 400);
            }
            $totalPuntos = $resultPointsTotal->data;

            $resultPointsWeekend = $this->nivelService->getPointsWeekendTotalByPlayer($idParticipante);
            if (!$resultPointsWeekend->success) {
                return response()->json(['error' => ['mensaje' => ["Error al consultar los puntos de la semana."]]], 400);
            }
            $puntoSemana = $resultPointsWeekend->data;

            $resultPointsMonth = $this->nivelService->getPointsMonthTotalByPlayer($idParticipante);
            if (!$resultPointsMonth->success) {
                return response()->json(['error' => ['mensaje' => ["Error al consultar los puntos del mes."]]], 400);
            }
            $puntosMes = $resultPointsMonth->data;

            $resultPointsCanjesTotal = $this->canjeService->getPointsCanjeByPlayer($idParticipante);
            if (!$resultPointsMonth->success) {
                return response()->json(['error' => ['mensaje' => ["Error al consultar los puntos de canje."]]], 400);
            }
            $puntosCanje = $resultPointsCanjesTotal->data;
            return response()->json([
                'total' => $totalPuntos,
                'puntoSemana' => $puntoSemana,
                'puntosMes' => $puntosMes,
                'puntosCanje' => $puntosCanje
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            Log::error('Error en el controller al consultar puntos: ' . $e->getMessage(), ['exception' => $e]);
            return response()->json(['error' => ['mensaje' => ["Error al consultar los puntos"]]], 400);
        }
    }

    /**
     * This PHP function retrieves the ranking of players based on a specified player type.
     * 
     * @param typePlayer The `getRankingByTypePlayers` function takes a parameter `` which
     * represents the type of player for which you want to retrieve the ranking. This function then
     * calls the `getPointsRankingByRolPlayer` method of the `playerService` to get the ranking based
     * on the
     * 
     * @return If the query is successful, the function will return the result of the points ranking
     * for the specified type of player. If the query is not successful, it will return a JSON response
     * with an error message indicating "Error al consultar el ranking" and a status code of 400.
     */
    public function getRankingByTypePlayers($typePlayer)
    {
        $resultQuery = $this->playerService->getPointsRankingByRolPlayer($typePlayer);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el ranking"]]], 400);
        }
        return $resultQuery->data;
    }

    /**
     * This PHP function retrieves insignias by participant ID and returns the data or an error message
     * if the query is unsuccessful.
     * 
     * @param idParticipante The `getInsigniasByParticipante` function takes an ``
     * parameter, which is used to retrieve insignias associated with a specific participant/player.
     * The function calls the `getInsigniasByPlayer` method from the `insigniaService` and returns the
     * 
     * @return the data obtained from the `getInsigniasByPlayer` method of the `insigniaService` for a
     * specific participant identified by the ``. If the query is successful, it returns
     * the data. If the query is not successful, it returns a JSON response with an error message
     * indicating "Error al consultar tus insignias" and a status code of
     */
    public function getInsigniasByParticipante($idParticipante)
    {
        $resultQuery = $this->insigniaService->getInsigniasByPlayer($idParticipante);
        if (!$resultQuery->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus insignias"]]], 400);
        }
        return $resultQuery->data;
    }

    public function getHistoryReportActionByPlayer($idParticipante)
    {
        $resultQueryRecompensas = $this->historialRecompensaService->getHistoryActionRecompensas($idParticipante);
        if(!$resultQueryRecompensas->success){
            return response()->json(['error' => ['mensaje' => ["Error al consultar tu historial de recompensas"]]], 400);
        }
        $collectionRecompensas = $resultQueryRecompensas->historyActions;
        $resultQueryCanjes = $this->canjeService->getHistoryActionCanjes($idParticipante);
        if (!$resultQueryCanjes->success) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus historial de canjes"]]], 400);
        }
        $collectionCanjes = $resultQueryCanjes->historyActions;
        $listHistoryCollection = $collectionRecompensas->merge($collectionCanjes)->sortByDesc('fecha');
        return $listHistoryCollection;
    }
}
