<?php

namespace App\Http\Controllers\V1\Seguridad;

use App\Dto\UserDto;
use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Http\Requests\LoginRequest;
use Illuminate\Http\Request;
use App\Services\AuthService;
use Tymon\JWTAuth\Exceptions\JWTException;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use App\Services\PlayerService;

class AuthController extends Controller
{
    protected $playerService;
    protected $authService;

    public function __construct(PlayerService $playerService, AuthService $authService)
    {
        $this->playerService = $playerService;
        $this->authService = $authService;
    }

    public function register(Request $request)
    {
        $data = $request->only('email', 'password', 'nombre');
        $validator = Validator::make($data, [
            'email' => 'required|email|unique:users',
            'password' => 'required|string|min:6|max:12',
            'nombre' => 'required|max:400|string',
        ]);
        if ($validator->fails()) {
            return response()->json($validator->messages(), 400);
        }
        try {
            $userDto = new UserDto();
            $userDto->email = $request->email;
            $userDto->name = $request->nombre;
            $userDto->password = $request->password;
            $resultRegister = $this->authService->registerUserRol($userDto, "administrador");
            if(!$resultRegister->success){
                return response()->json('Error al crear el usuario', 401);
            }
            return response()->json([
                'message' => 'Se registro con exito el usuario'
            ], Response::HTTP_OK);
        } catch (JWTException $e) {
            return response()->json('Error al registrar el usuario', 401);
        }
    }

    public function confirmarUsuario(Request $request)
    {
        $data = $request->only('password', 'password_confirmation', 'email');

        $validator = Validator::make($data, [
            'password' => 'required|confirmed|min:6',
            'email' => 'required|email|exists:users,email',
        ]);

        if ($validator->fails()) {
            return response()->json($validator->messages(), 400);
        }
        try {
            $resultUserFind = $this->authService->getUserByPasswordConfirm($request->email, false);
            if($resultUserFind){
                return response()->json('El usuario ya confirmo la contraseña o no existe el usuario', 401);
            }
            $resultUserFind->data->update([
                'password' => bcrypt($request->password),
                'password_confirmed' => true
            ]);
            return response()->json([
                'message' => 'La contraseña se confirmo con exito.'
            ], Response::HTTP_OK);
        } catch (JWTException $e) {
            return response()->json('Error al confirmar la contraseña', 401);
        }
    }

    /**
     * The function `authenticate` handles user authentication, session tracking, permission retrieval,
     * and user information retrieval in a PHP application.
     * 
     * @param LoginRequest request The `authenticate` function you provided seems to handle user
     * authentication in a Laravel application. It takes a `LoginRequest` object as a parameter.
     * 
     * @return The `authenticate` function returns a JSON response containing either an error message
     * with a status code of 400 or 401 if validation fails or authentication fails, respectively. If
     * authentication is successful, it returns a JSON response with a token, user information, and
     * player profile information. If an exception of type `JWTException` is caught during the process,
     * it returns an error message with a status code of
     */
    public function authenticate(LoginRequest $request)
    {
        if (isset($request->validator) && $request->validator->fails()) {
            return response()->json(['error' => $request->validator->messages()], 400);
        }

        $credentials = [
            "email" => $request->email,
            "password" => $request->password
        ];

        if (!$token = auth()->attempt($credentials)) {
            return response()->json([
                'message' => 'El usuario o contraseña son incorrectos',
            ], 401);
        }
        try {
            /* It could be used to track the number of sessions a user has open or
        perform some session-related logic. The exact implementation of the `check_session_count`
        method would be defined in the `Helper` class. */
            Helper::check_session_count(auth()->user());

            Auth::user()->getAllPermissions();

            $resultPlayerDetail = $this->playerService->getPlayerByUserId(Auth::user()->id);
            if (!$resultPlayerDetail->success) {
                return response()->json([
                    'message' => 'Error al consultar la información del usuario',
                ], 401);
            }

            return response()->json([
                'token' => $token,
                'user' => Auth::user(),
                "perfil" => $resultPlayerDetail->data
            ]);
        } catch (JWTException $e) {
            return response()->json([
                'message' => 'Hubo un error al autenticar.',
            ], 500);
        }
    }

    /**
     * The function logs out a user by invalidating the provided token and returns a JSON response
     * indicating success or failure.
     * 
     * @param Request request The `logout` function you provided is a PHP function that handles user
     * logout by invalidating the token. It takes a `Request` object as a parameter, which likely
     * contains the token that needs to be invalidated.
     * 
     * @return The `logout` function is returning a JSON response. If the validation fails, it returns
     * an error message with status code 400. If the token is successfully invalidated and the user is
     * logged out, it returns a success message with status code 200. If there is an exception during
     * the logout process, it returns an error message with status code 500.
     */
    public function logout(Request $request)
    {
        $validator = Validator::make($request->only('token'), [
            'token' => 'required|string'
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            auth()->setToken($request->token)->logout(true);
            return response()->json([
                'success' => true,
                'message' => 'User disconnected'
            ]);
        } catch (JWTException $exception) {
            return response()->json([
                'message' => 'Error al invalidar el token',
            ], 500);
        }
    }

    public function getUser(Request $request)
    {
        $this->validate($request, [
            'token' => 'required'
        ]);
        $user = auth()->setToken($request->token)->user();
        if (!$user) {
            return response()->json([
                'message' => 'Invalid token / token expired',
            ], 401);
        }
        return response()->json(['user' => $user]);
    }
}
