<?php

namespace App\Repositories;

use App\Dto\CanjeDto;
use App\Dto\HistoryActionPlayerDto;
use App\Interfaces\CanjeInterface;
use App\Models\Canje;
use App\Models\NivelReto;
use Carbon\Carbon;
use Illuminate\Support\Collection;

class CanjeRepository implements CanjeInterface
{
    public function allRecompensasCanjeadas()
    {
        $canjes = Canje::join('participantes', 'canjes.participante_id', "=", "participantes.id")
            ->with("recompensa.files")
            ->join('recompensas', 'canjes.recompensa_id', "=", "recompensas.id")
            ->join('files', 'recompensas.file_id', "=", "files.id")
            ->select(
                "canjes.id",
                "canjes.proceso_estatus",
                "canjes.puntos_canjeados",
                "canjes.fecha_canje",
                "canjes.numero_canje",
                "canjes.created_at",
                "participantes.nombre_completo",
                "recompensas.id as recompensa_id",
                "recompensas.file_id",
                "recompensas.nombre as nombre_recompensa",
                "recompensas.descripcion_premio",
                "recompensas.meta_canje"
            )
            ->orderBy("canjes.numero_canje", "asc")
            ->get();
        return $canjes;
    }

    public function findCanje(string $idCanje)
    {
        $canjeDetail = Canje::join('participantes', 'canjes.participante_id', "=", "participantes.id")
            ->with("recompensa.files")
            ->join('recompensas', 'canjes.recompensa_id', "=", "recompensas.id")
            ->join('files', 'recompensas.file_id', "=", "files.id")
            ->where("canjes.id", $idCanje)
            ->select(
                "canjes.id",
                "canjes.proceso_estatus",
                "canjes.puntos_canjeados",
                "canjes.fecha_canje",
                "canjes.numero_canje",
                "canjes.created_at",
                "participantes.nombre_completo",
                "recompensas.id as recompensa_id",
                "recompensas.nombre as nombre_recompensa",
                "recompensas.descripcion_premio",
                "recompensas.meta_canje",
                "recompensas.file_id"
            )
            ->orderBy("canjes.numero_canje", "asc")
            ->first();
        return $canjeDetail;
    }

    public function createCanje(CanjeDto $canjeDto)
    {
        $newCaje = Canje::create([
            "participante_id" => $canjeDto->participanteId,
            "recompensa_id" => $canjeDto->recompensaId,
            "puntos_canjeados" => $canjeDto->puntosCanjeados,
        ]);
        return $newCaje;
    }

    public function updateCanje(CanjeDto $canjeDto, string $idCanje)
    {
        $currentDateTime = Carbon::now();
        $dateNow = $currentDateTime->format('Y-m-d');
        $updateCanje = Canje::findOrFail($idCanje);
        $updateCanje->update([
            "proceso_estatus" => $canjeDto->procesoEstatus,
            "fecha_canje" => $dateNow
        ]);
        $canjeUpdateDto = new CanjeDto();
        $canjeUpdateDto->id = $updateCanje->id;
        $canjeUpdateDto->participanteId = $updateCanje->participante_id;
        $canjeUpdateDto->recompensaId = $updateCanje->recompensa_id;
        $canjeUpdateDto->puntosCanjeados = $updateCanje->puntos_canjeados;
        $canjeUpdateDto->numeroCanje = $updateCanje->numero_canje;
        return $canjeUpdateDto;
    }

    public function findRecompensasCanjes(string $idPlayer)
    {
        $canjes = Canje::join('participantes', 'canjes.participante_id', "=", "participantes.id")
            ->with("recompensa.files")
            ->where("canjes.participante_id", $idPlayer)
            ->join('recompensas', 'canjes.recompensa_id', "=", "recompensas.id")
            ->join('files', 'recompensas.file_id', "=", "files.id")
            ->select(
                "canjes.id",
                "canjes.proceso_estatus",
                "canjes.puntos_canjeados",
                "canjes.fecha_canje",
                "canjes.numero_canje",
                "canjes.created_at",
                "canjes.recompensa_id",
                "participantes.nombre_completo",
                "recompensas.id as recompensa_id",
                "recompensas.file_id",
                "recompensas.nombre as nombre_recompensa",
                "recompensas.descripcion_premio",
                "recompensas.meta_canje"
            )
            ->orderBy("canjes.numero_canje", "asc")
            ->get();
        return $canjes;
    }

    public function pointsTotalCanjesByPlayer(string $idPlayer)
    {
        $puntosAcumulados = NivelReto::whereHas('recompensa', function ($query) use ($idPlayer) {
            $query->where('historial_recompensas.participante_id', $idPlayer);
        })->get()->pluck('puntos')->sum();
        $puntosCanjeados = Canje::where("participante_id", $idPlayer)
            ->where("proceso_estatus", "!=", "rechazado")
            ->get()->pluck("puntos_canjeados")->sum();
        $totalPuntosCanjes = $puntosAcumulados - $puntosCanjeados;
        if ($totalPuntosCanjes < 0) {
            $totalPuntosCanjes = 0;
        }
        return $totalPuntosCanjes;
    }

    public function historyActionCanjesRecompensa(string $idPlayer){
        $canjesQuery = Canje::join("recompensas", 'canjes.recompensa_id', 'recompensas.id')
        ->where("canjes.proceso_estatus", "aprobado")
        ->where("canjes.participante_id", $idPlayer)
        ->select("canjes.puntos_canjeados as puntos", "canjes.fecha_canje as fecha", "recompensas.nombre as recompensa_nombre",
        "recompensas.descripcion_premio as descripcion_recompensa")
        ->get();

        $datos = $canjesQuery->map(function($canje){
            $newHistoryAction = new HistoryActionPlayerDto();
            $newHistoryAction->accion = "Usaste";
            $newHistoryAction->cantidadPuntos = $canje->puntos;
            $newHistoryAction->descripcion = "$canje->recompensa_nombre - $canje->descripcion_recompensa";
            $newHistoryAction->fecha = $canje->fecha;
            return $newHistoryAction;
        });
        return new Collection($datos);
    }
}
